#include "ArduinoESP.h"

ArduinoESP::ArduinoESP(uint8_t rx, uint8_t tx){
    static SoftwareSerial esp(rx,tx);
    mSerial = &esp;
    serialMonitor = &Serial;
}

void ArduinoESP::begin(long baudrate){
    mSerial -> begin(baudrate);
}


void ArduinoESP::checkStringIPD(String tempdata2){
  int findex=tempdata2.indexOf("+IPD,");
  if(findex>-1){
    found_http_IPD=true;
    connectionID=connectionID=char(tempdata2[findex+5]);  //.substring(findex+5,findex+6).toInt();
  }else{
    found_http_IPD=false;
    connectionID=0;
  }
}

void ArduinoESP::beginConfig(){
    unsigned long _timer;
    for(byte i = 0 ; i < 5 ; i++){
        mSerial -> begin(BAUD[i]);
        serialMonitor -> print(".");
        delay(50);
        while(mSerial -> available()) mSerial -> read();
        _timer = millis();
        while(millis() - _timer < TIME_OUT){
            mSerial -> print("AT\r\n");
            delay(50);
            if(mSerial -> available() > 0){
                if(response(false).indexOf("OK") != -1) {
                    serialMonitor -> println("\nESP8266 in AT Mode!");
                    serialMonitor -> println("Ready!");
                    AT_MODE = true;
                    break;
                }
            }
        }
        if(AT_MODE) break; 
    }
    if(!AT_MODE) {
        serialMonitor -> println("\nTry again after 5 seconds");
        delay(5000);
        asm volatile ("jmp 0"); 
    }
}
void ArduinoESP::setBaudrate(long baudrate){
    serialMonitor -> print("Set baudrate : ");
    serialMonitor -> println(baudrate);
    mSerial -> print("AT+UART_DEF=");
    mSerial -> print(baudrate);
    mSerial -> print(",8,1,0,0\r\n");
    String _response = response();
    if(_response.indexOf("OK")){
      mSerial -> begin(baudrate);
      while(mSerial -> available()) mSerial -> read();
    }
}

void ArduinoESP::setWifi(String ssid, String password){
    unsigned long _timer;
    mSerial -> print("AT+CWMODE=1\r\n");
    if(response(false).indexOf("OK")){
      mSerial -> print("AT+CWQAP\r\n");
      if(response(false).indexOf("OK")){
        _timer = millis();
        String _cmd = "AT+CWJAP=\""+ssid+"\",\""+password+"\"\r\n";
        mSerial -> print(_cmd);
        serialMonitor -> print("Connect Wifi : ");
        serialMonitor -> println(ssid);
        while(millis() - _timer < 8000){
            serialMonitor -> print(".");
            delay(500); 
        } 
        serialMonitor -> println();     
        response(true);
      }
    }
}

void ArduinoESP::setAP(String ssid, String password){
  unsigned long _timer;
  mSerial -> print("AT+CWMODE=2\r\n");
  String _openAP= "AT+CWSAP=\""+ssid+"\",\""+password+"\",5,3\r\n";
  if(response(false).indexOf("OK") != -1){
    serialMonitor -> print("Open access point : ");
    serialMonitor -> print(ssid);
    serialMonitor -> print("  ");
    serialMonitor -> println(password);
    mSerial -> print(_openAP);
    _timer = millis();
    while(millis() - _timer < 4000){
      serialMonitor -> print(".");
      delay(500);
    }
    serialMonitor -> println();
    response(true);    
  }
}

void ArduinoESP::getLocalIP(){
    mSerial -> print("AT+CIFSR\r\n");
    delay(2000);
    //response(true);  
	String _data = "";
	while(mSerial -> available() > 0 ){
		_data += (char) mSerial -> read();
	}
	my_LocalIP="";
	for(int i=24 ; i<40 ; i++){
		my_LocalIP +=_data[i];
	}
	Serial.println("my_LocalIP=" + _data);
}

void ArduinoESP::createPort(int port){
  mSerial -> print("AT+CIPMUX=1\r\n");
   if(response(false).indexOf("OK") != -1){
    serialMonitor -> print("Create sever port : ");
    serialMonitor -> println(port);
    mSerial -> print("AT+CIPSERVER=1,");
    mSerial -> print(port);
    mSerial -> print("\r\n");
    response(true);
  }
}

void ArduinoESP::sendAT(String ATCommand){   
    mSerial -> print(ATCommand);
    mSerial -> print("\r\n");
    mSerial -> flush();
    response(true);
}

void ArduinoESP::sendData(int id, String data){
  mSerial -> print("AT+CIPSEND=");
  mSerial -> print(id);
  mSerial -> print(",");
  mSerial -> print(data.length());
  mSerial -> print("\r\n");
  delay(1);
  mSerial -> print(data);
  delay(20);
}

void ArduinoESP::sendDataSerial(int id, String data){
  mSerial -> print("AT+CIPSEND=");
  mSerial -> print(id);
  mSerial -> print(",");
  mSerial -> print(data.length());
  mSerial -> print("\r\n");
  delay(1);
  mSerial -> print(data);
  delay(20);

  mSerial -> print("AT+CIPCLOSE=");
  mSerial -> print(id);
  mSerial -> print("\r\n");
  delay(200);
}

String ArduinoESP::getDataSerial(){
  String _cmd = getData();
  int _begin = _cmd.indexOf(":");
  _cmd = _cmd.substring(_begin+1);
  return _cmd;
}

String ArduinoESP::getData(){
  String _data = "";
  while(mSerial -> available() > 0 ){
    _data += (char) mSerial -> read();
  }
  _data.trim();
  return _data;
}

String ArduinoESP::getRawDataSerial(){
  String _data = "";
  while(mSerial -> available() > 0 ){
    _data += (char) mSerial -> read();
	delay(20);
  }
  return _data;
}


String ArduinoESP::response(bool debug){
    String _response = "";
    while(true){
      delay(100);
        if ( mSerial -> available() > 0){
            _response += getData();
            if(debug) serialMonitor -> println(_response);
            break;
        }
    }
    return _response;
}

bool ArduinoESP::isDataArrive(){
  return mSerial -> available();
}

void ArduinoESP::setupThingSpeak(String channelID,String writeAPI,String readAPI){
  _channelID = channelID;
  _writeAPI = writeAPI;
  _readAPI = readAPI;
  mSerial -> print("AT+CIPMUX=1\r\n");
  delay(500);
}

void ArduinoESP::connectThingSpeak(){
  mSerial -> print(HOST);
  delay(1500);
  response(true);
}

void ArduinoESP::disconnectThingSpeak(){
  mSerial ->print("AT+CIPCLOSE\r\n");
  delay(500);
}

String ArduinoESP::subDataChannel(String tmp){
  int _start = tmp.indexOf(":");
  int _end = tmp.indexOf(",",_start);
  return tmp.substring(_start+1,_end);
}

void ArduinoESP::sendDataToChannel(String value1,String value2,String value3,String value4,String value5,String value6,String value7,String value8){
    connectThingSpeak();
    String _cmd = "GET https://api.thingspeak.com/update?api_key="+_writeAPI;
    if(value1 != "NULL") _cmd += "&field1=" + value1;
    if(value2 != "NULL") _cmd += "&field2=" + value2;
    if(value3 != "NULL") _cmd += "&field3=" + value3;
    if(value4 != "NULL") _cmd += "&field4=" + value4;
    if(value5 != "NULL") _cmd += "&field5=" + value5;
    if(value6 != "NULL") _cmd += "&field6=" + value6;
    if(value7 != "NULL") _cmd += "&field7=" + value7;
    if(value8 != "NULL") _cmd += "&field8=" + value8;
    _cmd += "\r\n";
    sendData(0,_cmd);
    delay(100);
    response();
}

float ArduinoESP::getDataFromChannel(int field){
  String _data;
  float _value;
  connectThingSpeak();
  String _cmd = "GET https://api.thingspeak.com/channels/"+_channelID+"/fields/"+String(field)+"/last?api_key=" +_readAPI+"\r\n";
  sendData(0,_cmd);
  delay(1000);
  _data = response(true);
  _data = subDataChannel(_data);
  _value = _data.toFloat();
  return _value;
}
